#include <WiFi.h>
#include <PubSubClient.h>
#include <Wire.h>

#include "config.h"
#define SLEEP_TIME 24 * 60 * 60 * 1000000
#define WAKEUP_PIN GPIO_NUM_27
#define BATT_PIN GPIO_NUM_35

WiFiClient espClient;
PubSubClient client(espClient);

void deepsleep()
{
    Serial.println("Deepsleep ...");
    esp_sleep_enable_timer_wakeup(SLEEP_TIME);

    if(digitalRead(WAKEUP_PIN))
    {
        esp_sleep_enable_ext0_wakeup(WAKEUP_PIN, 0);
    }
    else
    {
        esp_sleep_enable_ext0_wakeup(WAKEUP_PIN, 1);
    }

    esp_deep_sleep_start();
}

void setup_wifi() 
{
    Serial.print("Connecting to ");
    Serial.println(WLAN_SSID);

    WiFi.begin(WLAN_SSID, WLAN_PASS);

    int attempts = 10;
    while (WiFi.status() != WL_CONNECTED) 
    {
        delay(500);
        Serial.print(".");

        attempts--;
        if(attempts == 0)
        {
            Serial.println("WLAN failed");
            deepsleep();
        }
    }

    Serial.println("WiFi connected");
    Serial.println("IP address: ");
    Serial.println(WiFi.localIP());
}

void setup_mqtt() 
{
    client.setServer(MQTT_BROKER, 1883);

    Serial.println("Connecting MQTT ...");
    String clientId = "FABDETECT-";
    clientId += String(random(0xffff), HEX);

    Serial.print("Attempting MQTT connection...");
    if(MQTT_USERNAME == "")
    {
        client.connect(clientId.c_str());
    }
    else
    {
        client.connect(clientId.c_str(), MQTT_USERNAME, MQTT_PASSWORD);
    }

    if(!client.connected())
    {
        Serial.println("MQTT failed");
        Serial.println(client.state());
        deepsleep();
    }
    else
    {
        Serial.println("MQTT connected");
    }
}

void publish_msg()
{
    char message[20];
    
    sprintf(message, "%d", digitalRead(WAKEUP_PIN));
    char topic_pin[] = "fabdetect/00000/state";
    sprintf(topic_pin, "fabdetect/%05d/state", FABDETECTID);
    Serial.println(topic_pin);
    Serial.println(message);
    Serial.println(client.publish(topic_pin, message));

    float voltage = ((analogRead(BATT_PIN) * 3.3 ) / (4095)) * 2;

    sprintf(message, "%.2f", voltage);
    char topic_batt[] = "fabdetect/00000/battery";
    sprintf(topic_batt, "fabdetect/%05d/battery", FABDETECTID);
    Serial.println(topic_batt);
    Serial.println(message);
    Serial.println(client.publish(topic_batt, message));

    delay(1000);
}

void setup()
{
    Serial.begin(115200);

    setup_wifi();
    setup_mqtt();
    
    pinMode(WAKEUP_PIN, INPUT_PULLUP);
    pinMode(BATT_PIN, INPUT);

    publish_msg();
    deepsleep();
}

void loop() 
{
    client.loop();
}
